@@ -4,12 +4,16 @@ import shlex
 import shutil
 import stat
 from pathlib import Path
-from subprocess import check_output, run
+from subprocess import check_output, run, PIPE
 
-__all__ = ("get_runtime_library_path", "create_wine_bin_dir", "run_command")
+__all__ = ("get_runtime_library_paths", "create_wine_bin_dir", "run_command")
 
 logger = logging.getLogger("protontricks")
 
+SUPPORTED_STEAM_RUNTIMES = [
+    "Steam Linux Runtime - Soldier"
+]
+
 
 def lower_dict(d):
     """
@@ -21,20 +25,61 @@ def lower_dict(d):
     return {k.lower(): v for k, v in d.items()}
 
 
-def get_runtime_library_path(steam_runtime_path, proton_app):
+def get_host_library_paths():
+    """
+    Get host library paths to use when creating the LD_LIBRARY_PATH environment
+    variable for use with newer Steam Runtime installations
+    """
+    # The traditional Steam Runtime does the following when running the
+    # `run.sh --print-steam-runtime-library-paths` command.
+    # Since that command is unavailable with newer Steam Runtime releases,
+    # do it ourselves here.
+    result = run(
+        ["/sbin/ldconfig", "-XNv"],
+        check=True, stdout=PIPE, stderr=PIPE
+    )
+    lines = result.stdout.decode("utf-8").split("\n")
+    paths = [
+        line.split(":")[0] for line in lines
+        if line.startswith("/") and ":" in line
+    ]
+
+    return ":".join(paths)
+
+
+def get_runtime_library_paths(steam_runtime_path, proton_app):
     """
     Get LD_LIBRARY_PATH value to run a command using Steam Runtime
     """
+    if proton_app.required_tool_app:
+        # bwrap based Steam Runtime is used for Proton installations that
+        # use separate Steam runtimes
+        # TODO: Try to run the Wine binaries inside an user namespace somehow.
+        # Newer Steam Runtime environments may rely on a newer glibc than what
+        # is available on the host system, which may cause potential problems
+        # otherwise.
+        runtime_root = next(
+            proton_app.required_tool_app.install_path.glob("*/files/")
+        )
+        return "".join([
+            str(proton_app.install_path / "dist" / "lib"), os.pathsep,
+            str(proton_app.install_path / "dist" / "lib64"), os.pathsep,
+            get_host_library_paths(), os.pathsep,
+            str(runtime_root / "i686-pc-linux-gnu" / "lib"), os.pathsep,
+            str(runtime_root / "x86_64-pc-linux-gnu" / "lib")
+        ])
+
+    # Traditional LD_LIBRARY_PATH based Steam Runtime is used otherwise
     steam_runtime_paths = check_output([
-        os.path.join(steam_runtime_path, "run.sh"),
+        str(steam_runtime_path / "run.sh"),
         "--print-steam-runtime-library-paths"
     ])
     steam_runtime_paths = str(steam_runtime_paths, "utf-8")
     # Add Proton installation directory first into LD_LIBRARY_PATH
     # so that libwine.so.1 is picked up correctly (see issue #3)
     return "".join([
-        os.path.join(proton_app.install_path, "dist", "lib"), os.pathsep,
-        os.path.join(proton_app.install_path, "dist", "lib64"), os.pathsep,
+        str(proton_app.install_path / "dist" / "lib"), os.pathsep,
+        str(proton_app.install_path / "dist" / "lib64"), os.pathsep,
         steam_runtime_paths
     ])
 
@@ -53,9 +98,7 @@ def create_wine_bin_dir(proton_app):
     using Steam Runtime and Proton's own libraries instead of the system
     libraries
     """
-    binaries = list((
-        Path(proton_app.install_path) / "dist" / "bin"
-    ).iterdir())
+    binaries = list((proton_app.install_path / "dist" / "bin").iterdir())
 
     # Create the base directory containing files for every Proton installation
     xdg_cache_dir = os.environ.get(
@@ -67,7 +110,7 @@ def create_wine_bin_dir(proton_app):
     # Create a directory to hold the new executables for the specific
     # Proton installation
     bin_path = base_path / proton_app.name / "bin"
-    os.makedirs(str(bin_path), exist_ok=True)
+    bin_path.mkdir(parents=True, exist_ok=True)
 
     logger.info(
         "Created Steam Runtime Wine binary directory at %s", str(bin_path)
@@ -87,19 +130,19 @@ def create_wine_bin_dir(proton_app):
     # longer exist in the Proton installation, so we'll also get rid of
     # scripts that point to non-existing files
     shutil.rmtree(str(bin_path))
-    os.makedirs(str(bin_path))
+    bin_path.mkdir(parents=True)
 
     for binary in binaries:
         content = WINE_SCRIPT_TEMPLATE.format(
             name=shlex.quote(binary.name)
         ).encode("utf-8")
 
-        with open(str(bin_path / binary.name), "wb") as file_:
-            file_.write(content)
+        proxy_script_path = bin_path / binary.name
+        proxy_script_path.write_bytes(content)
 
-        script_stat = os.stat(str(bin_path / binary.name))
+        script_stat = proxy_script_path.stat()
         # Make the helper script executable
-        (bin_path / binary.name).chmod(script_stat.st_mode | stat.S_IEXEC)
+        proxy_script_path.chmod(script_stat.st_mode | stat.S_IEXEC)
 
     return bin_path
 
@@ -124,48 +167,61 @@ def run_command(
             "WINE environment variable is not available. "
             "Setting WINE environment variable to Proton bundled version"
         )
-        os.environ["WINE"] = os.path.join(
-            proton_app.install_path, "dist", "bin", "wine")
+        os.environ["WINE"] = \
+            str(proton_app.install_path / "dist" / "bin" / "wine")
 
     if not os.environ.get("WINESERVER"):
         logger.info(
             "WINESERVER environment variable is not available. "
             "Setting WINESERVER environment variable to Proton bundled version"
         )
-        os.environ["WINESERVER"] = os.path.join(
-            proton_app.install_path, "dist", "bin", "wineserver"
-        )
+        os.environ["WINESERVER"] = \
+            str(proton_app.install_path / "dist" / "bin" / "wineserver")
 
-    os.environ["WINETRICKS"] = winetricks_path
-    os.environ["WINEPREFIX"] = steam_app.prefix_path
+    os.environ["WINETRICKS"] = str(winetricks_path)
+    os.environ["WINEPREFIX"] = str(steam_app.prefix_path)
     os.environ["WINELOADER"] = os.environ["WINE"]
     os.environ["WINEDLLPATH"] = "".join([
-        os.path.join(proton_app.install_path, "dist", "lib64", "wine"),
+        str(proton_app.install_path / "dist" / "lib64" / "wine"),
         os.pathsep,
-        os.path.join(proton_app.install_path, "dist", "lib", "wine")
+        str(proton_app.install_path / "dist" / "lib" / "wine")
     ])
 
-    os.environ["PATH"] = os.path.join(
-        proton_app.install_path, "dist", "bin"
-    ) + os.pathsep + os.environ["PATH"]
+    os.environ["PATH"] = "".join([
+        str(proton_app.install_path / "dist" / "bin"), os.pathsep,
+        os.environ["PATH"]
+    ])
 
     # Expose the path to Proton installation. This is mainly used for
     # Wine helper scripts, but other scripts could use it as well.
-    os.environ["PROTON_PATH"] = proton_app.install_path
+    os.environ["PROTON_PATH"] = str(proton_app.install_path)
 
     # Unset WINEARCH, which might be set for another Wine installation
     os.environ.pop("WINEARCH", "")
 
     wine_bin_dir = None
     if steam_runtime_path:
+        if proton_app.required_tool_app:
+            runtime_name = proton_app.required_tool_app.name
+            logger.info(
+                "Using separately installed Steam Runtime: %s",
+                runtime_name
+            )
+
+            if runtime_name not in SUPPORTED_STEAM_RUNTIMES:
+                logger.warning(
+                    "Current Steam Runtime not recognized by Protontricks."
+                )
+
         # When Steam Runtime is enabled, create a set of helper scripts
         # that load the underlying Proton Wine executables with Steam Runtime
         # and Proton libraries instead of system libraries
         wine_bin_dir = create_wine_bin_dir(proton_app=proton_app)
         os.environ["PROTON_LD_LIBRARY_PATH"] = \
-            get_runtime_library_path(steam_runtime_path, proton_app)
-        os.environ["PATH"] = \
-            str(wine_bin_dir) + os.pathsep + os.environ["PATH"]
+            get_runtime_library_paths(steam_runtime_path, proton_app)
+        os.environ["PATH"] = "".join([
+            str(wine_bin_dir), os.pathsep, os.environ["PATH"]
+        ])
         os.environ["WINE"] = str(wine_bin_dir / "wine")
         os.environ["WINELOADER"] = os.environ["WINE"]
         os.environ["WINESERVER"] = str(wine_bin_dir / "wineserver")
